package com.dayou.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.dayou.common.BaseEntity;
import com.dayou.dto.FinanceFineSelectDTO;
import com.dayou.dto.SimplePostModel;
import com.dayou.entity.FinanceFine;
import com.dayou.entity.FinanceFineRefund;
import com.dayou.exception.ErrorCode;
import com.dayou.mapper.FinanceFineMapper;
import com.dayou.service.IFinanceFineRefundService;
import com.dayou.service.IFinanceFineService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.dayou.utils.DateUtils;
import com.dayou.utils.LoginContext;
import com.dayou.vo.CompileFineAssetsOrder;
import com.dayou.vo.CompileFineMajorOrder;
import com.dayou.vo.FinanceFineVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.springframework.transaction.annotation.Transactional;

/**
 * <p>
 * 超期未收款罚款表 服务实现类
 * </p>
 *
 * @author GouGengquan
 * @since 2025-04-07
 */
@Service
public class FinanceFineServiceImpl extends ServiceImpl<FinanceFineMapper, FinanceFine> implements IFinanceFineService {

    @Autowired
    private FinanceFineMapper financeFineMapper;

    @Autowired
    private IFinanceFineRefundService financeFineRefundService;

    /**
     * 分页查询超期罚款
     *
     * @param page                 分页
     * @param financeFineSelectDTO 查询条件dto
     * @return Page<FinanceFineVO>
     */
    @Override
    public Page<FinanceFineVO> selectPage(Page page, FinanceFineSelectDTO financeFineSelectDTO) {
        // 此处数据权限,客户经理只能查看自己的项目
        if (LoginContext.getLoginCacheUserBO().getPostList().stream()
                .map(SimplePostModel::getName)
                .anyMatch(name -> name.contains("客户经理") || name.contains("销售代表"))) {
            financeFineSelectDTO.setClientManagerId(LoginContext.getUserId());
        }
        return financeFineMapper.selectPage(page, financeFineSelectDTO);
    }

    /**
     * 导出查询超期罚款
     *
     * @param financeFineSelectDTO 查询条件dto
     * @return List<FinanceFineVO>
     */
    @Override
    public List<FinanceFineVO> exportList(FinanceFineSelectDTO financeFineSelectDTO) {
        // 此处数据权限,客户经理只能查看自己的项目
        if (LoginContext.getLoginCacheUserBO().getPostList().stream()
                .map(SimplePostModel::getName)
                .anyMatch(name -> name.contains("客户经理") || name.contains("销售代表"))) {
            financeFineSelectDTO.setClientManagerId(LoginContext.getUserId());
        }
        return financeFineMapper.exportList(financeFineSelectDTO);
    }

    /**
     * 取消确认罚款金额
     *
     * @return Boolean
     */
    @Override
    public Boolean cancelFineAmount(Long id) {
        // 判断罚款记录是否已产生退款记录
        int refundCount = financeFineRefundService.count(new LambdaQueryWrapper<FinanceFineRefund>().eq(FinanceFineRefund::getFineId, id).eq(BaseEntity::getDeleted, 0));
        if (refundCount > 0) {
            ErrorCode.throwBusinessException(ErrorCode.CUSTOM_ERROR,"当前罚款记录已产生相应退款记录, 不可取消罚款!");
        }
        return financeFineMapper.cancelFineAmount(id);
    }

    /**
     * 批量取消确认罚款金额
     * @return Boolean
     */
    @Override
    public Boolean batchCancelFineAmount(List<Long> ids) {
        // 判断罚款记录是否已产生退款记录
        int refundCount = financeFineRefundService.count(new LambdaQueryWrapper<FinanceFineRefund>().in(FinanceFineRefund::getFineId, ids).eq(BaseEntity::getDeleted, 0));
        if (refundCount > 0) {
            ErrorCode.throwBusinessException(ErrorCode.CUSTOM_ERROR,"当前罚款记录已产生相应退款记录, 不可取消罚款!");
        }
        if (!ids.isEmpty()) {
            return financeFineMapper.batchCancelFineAmount(ids);
        }else {
            ErrorCode.throwBusinessException(ErrorCode.CUSTOM_ERROR,"未选择罚款记录!");
        }
        return Boolean.FALSE;
    }


    @Override
    public FinanceFine detail(Long id) {
        return this.getById(id);
    }

    @Override
    public Boolean add(FinanceFine financeFine) {
        return this.save(financeFine);
    }

    @Override
    public Boolean update(FinanceFine financeFine) {
        // 判断罚款记录是否已产生退款记录
        int refundCount = financeFineRefundService.count(new LambdaQueryWrapper<FinanceFineRefund>().eq(FinanceFineRefund::getFineId, financeFine.getId()).eq(BaseEntity::getDeleted, 0));
        if (refundCount > 0) {
            ErrorCode.throwBusinessException(ErrorCode.CUSTOM_ERROR,"当前罚款记录已产生相应退款记录, 不可修改罚款信息!");
        }
        // 已确认的罚款记录修改罚款金额时同步修改实际罚款金额
        if (ObjectUtil.isNotNull(financeFine.getFineAmount()) && financeFine.getStatus()) {
            financeFine.setRealFineAmount(financeFine.getFineAmount());
        }
        return this.updateById(financeFine);
    }

    @Override
    public Boolean delete(Long id) {
        //逻辑删除
        return this.removeById(id);
    }

    /**
     * 统计财务收款预警中超过270天的订单或产品新增到罚款表
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void compileFineOrder() {
        List<CompileFineAssetsOrder> compileFineAssetsOrderList = financeFineMapper.getAssetsOrder();
        List<Long> compileFinePersonalOrderList = financeFineMapper.getPersonalOrder();
        List<CompileFineMajorOrder> compileFineMajorOrderList = financeFineMapper.getMajorOrder();

        List<FinanceFine> financeFineList = new ArrayList<>();

        // 处理资产的信息以插入到罚款表
        for (CompileFineAssetsOrder assetsOrder : compileFineAssetsOrderList) {
            // 获取开票金额的5%作为罚款金额
            BigDecimal fineAmount = financeFineMapper.getInvoiceByFundInfo(assetsOrder.getOrderFundId(), assetsOrder.getProductionFundId());
            // 判断罚款金额是否为null(为null说明没开票,通过其他信息设置罚款金额)
            if (ObjectUtil.isNull(fineAmount)) {
                // 判断应收款是否不为null且大于1(乱填)
                if (ObjectUtil.isNotNull(assetsOrder.getProductionShouldAmount()) && assetsOrder.getProductionShouldAmount().compareTo(BigDecimal.valueOf(1)) > 0) {
                    // 填了应收款,按应收款的5%
                    fineAmount = assetsOrder.getProductionShouldAmount().multiply(BigDecimal.valueOf(0.05));
                } else {
                    // 没有应收款的话,罚款金额就是 standard_amount * 30% * 5%
                    if (ObjectUtil.isNotNull(assetsOrder.getStandardAmount())) {
                        fineAmount = assetsOrder.getStandardAmount().multiply(BigDecimal.valueOf(0.3)).multiply(BigDecimal.valueOf(0.05));
                    } else {
                        fineAmount = BigDecimal.valueOf(0);
                    }
                }
            }

            // 最低罚款:200元,不⾜200元,按200元罚款 最⾼罚款⾦额:5000元,超出5000元,按5000元罚款
            // 判断罚款金额是否大于 200 小于5000
            // 返回值大于0:第一个BigDecimal大于第二个
            if (fineAmount.compareTo(BigDecimal.valueOf(200)) < 0) { // 金额小于200
                fineAmount = BigDecimal.valueOf(200);
            } else if (fineAmount.compareTo(BigDecimal.valueOf(5000)) > 0) { // 金额大于5000
                fineAmount = BigDecimal.valueOf(5000);
            }

            // 设置罚款信息
            FinanceFine financeFine = new FinanceFine();
            financeFine.setOrderFundId(assetsOrder.getOrderFundId());
            financeFine.setProductionFundId(assetsOrder.getProductionFundId());
            financeFine.setFineAmount(fineAmount);
            // 2025-05-23日,修改数据库确认字段默认为已确认,所以新增日期就是确认罚款日期,且实际罚款金额直接设置为罚款金额
            financeFine.setConfirmDate(DateUtils.dateToLocalDateTime(new Date()));
            financeFine.setRealFineAmount(fineAmount);
            financeFineList.add(financeFine);
        }

        // 处理个贷的信息以插入到罚款表
        for (Long orderFundId : compileFinePersonalOrderList) {
            // 设置罚款信息
            FinanceFine financeFine = new FinanceFine();
            financeFine.setOrderFundId(orderFundId);
            financeFine.setFineAmount(BigDecimal.valueOf(50));
            // 2025-05-23日,修改数据库确认字段默认为已确认,所以新增日期就是确认罚款日期,且实际罚款金额直接设置为罚款金额
            financeFine.setConfirmDate(DateUtils.dateToLocalDateTime(new Date()));
            financeFine.setRealFineAmount(BigDecimal.valueOf(50));
            financeFineList.add(financeFine);
        }

        // 处理大中型的信息以插入到罚款表
        for (CompileFineMajorOrder majorOrder : compileFineMajorOrderList) {
            // 获取开票金额的5%作为罚款金额
            BigDecimal fineAmount = financeFineMapper.getInvoiceByFundInfo(majorOrder.getOrderFundId(), majorOrder.getProductionFundId());
            // 判断罚款金额是否为null(为null说明没开票,通过其他信息设置罚款金额)
            if (ObjectUtil.isNull(fineAmount)) {
                // 判断应收款是否不为null且不等于1(乱填)
                if (ObjectUtil.isNotNull(majorOrder.getProductionShouldAmount()) && majorOrder.getProductionShouldAmount().compareTo(BigDecimal.valueOf(1)) > 0) {
                    // 填了应收款,按应收款的5%
                    fineAmount = majorOrder.getProductionShouldAmount().multiply(BigDecimal.valueOf(0.05));
                } else {
                    // 没有应收款的话,罚款金额就是 standard_amount * 30% * 5%
                    if (ObjectUtil.isNotNull(majorOrder.getStandardAmount())) {
                        fineAmount = majorOrder.getStandardAmount().multiply(BigDecimal.valueOf(0.3)).multiply(BigDecimal.valueOf(0.05));
                    } else {
                        fineAmount = BigDecimal.valueOf(0);
                    }
                }
            }

            // 最低罚款:200元,不⾜200元,按200元罚款 最⾼罚款⾦额:5000元,超出5000元,按5000元罚款
            // 判断罚款金额是否大于 200 小于5000
            // 返回值大于0:第一个BigDecimal大于第二个
            if (fineAmount.compareTo(BigDecimal.valueOf(200)) < 0) { // 金额小于200
                fineAmount = BigDecimal.valueOf(200);
            } else if (fineAmount.compareTo(BigDecimal.valueOf(5000)) > 0) { // 金额大于5000
                fineAmount = BigDecimal.valueOf(5000);
            }

            // 设置罚款信息
            FinanceFine financeFine = new FinanceFine();
            financeFine.setOrderFundId(majorOrder.getOrderFundId());
            financeFine.setProductionFundId(majorOrder.getProductionFundId());
            financeFine.setFineAmount(fineAmount);
            // 2025-05-23日,修改数据库确认字段默认为已确认,所以新增日期就是确认罚款日期,且实际罚款金额直接设置为罚款金额
            financeFine.setConfirmDate(DateUtils.dateToLocalDateTime(new Date()));
            financeFine.setRealFineAmount(fineAmount);
            financeFineList.add(financeFine);
        }

        this.saveBatch(financeFineList);
    }

    /**
     * 根据收款信息获取符合条件的罚款表信息
     * 且已确认罚款
     *
     * @param orderFundId      订单收款id
     * @param productionFundId 产品收款id
     * @return FinanceFine
     */
    @Override
    public FinanceFine getFineByFundInfo(Long orderFundId, Long productionFundId) {
        return financeFineMapper.getFineByFundInfo(orderFundId, productionFundId);
    }

    @Override
    public Boolean confirmFineAmount(Long id) {
        return financeFineMapper.confirmFineAmount(id);
    }


}
